/*
 *  Stefan's Portable MakeTurn: "General Purpose" memory buffer
 *
 *  (c) 1999 Stefan Reuther <Streu@gmx.de>
 */

#include <algorithm>
#include "buffer.h"

using namespace std;

/* construct a new buffer */
CBuffer::CBuffer()
    : buffer(new char[128]), length(0), capacity(128)
{ }

CBuffer::~CBuffer()
{
    delete[] buffer;
}

/* ensure that all elements in [0,cap) are accessible,
   length may grow up to this value */
void CBuffer::EnsureCapacity(size_t cap)
{
    if(capacity >= cap)
        return;
    if(cap < length*2)
        cap = length*2;        

    char* n = new char[cap];
    copy(buffer, buffer + length, n);
    delete[] buffer;
    buffer = n;
    capacity = cap;
}

/* append LEN bytes from VAL */
void CBuffer::Append(const char* val, size_t len)
{
    EnsureCapacity(length + len);
    copy(val, val + len, buffer + length);
    length += len;
}

/* append little-endian word */
void CBuffer::AppendWord(int n)
{
    char x[2];
    x[0] = n;
    x[1] = n >> 8;
    Append(x, 2);
}

/* append little-endian long */
void CBuffer::AppendLong(long n)
{
    char x[4];
    x[0] = n;
    x[1] = n >> 8;
    x[2] = n >> 16;
    x[3] = n >> 24;
    Append(x, 4);
}

/* get a word */
int CBuffer::GetWord(size_t n)
{
    unsigned char* p = (unsigned char*)(buffer + n);
    return p[0] + 256UL*p[1];
}

/* get a long */
long CBuffer::GetLong(size_t n)
{
    unsigned char* p = (unsigned char*)(buffer + n);
    return p[0] + 256UL*p[1] + 65536UL*p[2] + 16777216*p[3];
}

/* sum up all values */
unsigned long CBuffer::Sum()
{
    unsigned long l = 0;
    for(size_t i = 0; i < length; i++)
        l += (unsigned char)(buffer[i]);
    return l;
}
