/*
 *  Stefan's Portable MakeTurn: Main Module
 *
 *  (c) 1999,2002,2004 Stefan Reuther <Streu@gmx.de>
 */
#include <iostream>
#include <string>
#include <cstdio>
#include "turnfile.h"
#include "generate.h"

using namespace std;

char* progname;

bool phost_only = false;
bool thost_only = false;
bool winplan_turn = false;

/* exit with error message */
static void error_exit(const char* msg)
{
    cerr << progname << ": " << msg << endl;
    exit(1);
}

int main(int argc, char** argv)
{
    char* directory = 0;

    progname = *argv++;
    cout << "Stefan's Portable MakeTurn - (c) 1999-2004 Stefan Reuther <Streu@gmx.de>" << endl << flush;
    while(--argc) {
        if(**argv == '-') {
            char* p = *argv;
            while (*++p) {
                switch(*p) {
                 case 'p':
                    phost_only = true;
                    thost_only = false;
                    break;
                 case 't':
                    thost_only = true;
                    phost_only = false;
                    break;
                 case 'w':
                    winplan_turn = true;
                    break;
                 default:
                    error_exit("unknown option");
                }
            }
        } else {
            if(!directory)
                directory = *argv;
            else
                error_exit("too many directory names");
        }
        argv++;
    }
    if(!directory)
        error_exit("please specify a directory name");

    /* load fizz.bin */
    try {
        LoadRegistration(directory);
    }
    catch(string s) {
        error_exit(s.c_str());
    }
    
    /* generate turn files */
    CTurnFile* turnfiles[12];
    long templock[12];
    for(int i = 1; i <= 11; i++) {
        templock[i] = 0;
        try {
            turnfiles[i] = GenerateTurnCommands(directory, i);
            if(turnfiles[i]) {
                if (winplan_turn)
                    turnfiles[i]->MakeWinplanTrailer();
                templock[i] = turnfiles[i]->GenerateHeader();
            }
        }
        catch(string s) {
            turnfiles[i] = 0;
            cerr << progname << ": " << s << endl;
        }
    }

    /* now write them out */
    for(int i = 1; i <= 11; i++)
        if(turnfiles[i]) {
            char name[20];
            sprintf(name, "/player%d.trn", i);
            string name_str = string(directory) + name;
            
            turnfiles[i]->Write(name_str, templock);
        }

    cout << "done." << endl;
}
